/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOTIFY_H
#define _GNOTIFY_H

#include <gtk/gtk.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <signal.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#define VERSION "0.7"
#define GNOTIFY_ICON "/usr/share/icons/gnome/32x32/apps/web-browser.png"

/* The config-struct. Here all information gnotify got from the 
 * ~/.gnotify.xml will be stored */
struct CONF
{
  int notify_x;
  int notify_y;
  int notify_delay;
  int notify_play;
  int notify_ipdisallows;
	int notify_forkgtk;
	int notify_allowmove;
  char notify_ssink[8];
  char notify_allowip[128][32];
}cfg;

 /* The Client-Database. Every client gehts one array of this struct
	* and will use it for storing things like his IP, his Socket-
	* Filedescriptor and other stuff. */
struct CLIENTSDB
{
	pthread_t th;
	int sockfd;
	int move_x;
	int move_y;
	char ip[32];
  char app[64];
  char icon[256];
  char snd[256];
  char msg[512];
  char button[32];
}cdb[128];

pthread_mutex_t mutex; /* This is the mutex which is used for a thread-
												* synchronization. */

void gnotify_main(void);
void gnotify_about(void);
void gnotify_atexit(void);

/* This is the gnotify-signal-handler.
 * Parameters:
 * sign - The signal which arrives
 */
void gnotify_signal(int sign);

/* The gnotify-clienthandling function,
 * Parameters:
 * cid - The client-id (needed for accessing the cdb[])
 */
int gnotify_client(int cid);

int gnotify_notify(int cid);
static void gnotify_notify_quit(GtkWidget *widget,gpointer data);
static void buttonclick(GtkWidget *widget,gpointer data);
int playsound(const char *file);

/* This function gets a free Client-ID from the cdb-struct and returns it */
int cid_getfree(void);

char *reverse_strnget(const char* string1, int n);
int readconfig(char *config);
int createconfig(char *config);

#endif
