/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "include/gnotify.h"

int sockett;

int main(int argc, char *argv[])
{
  char buff[128];
  
	if(argc>=2)
  {
	  if(strcmp(argv[1],"-h")==0||strcmp(argv[1],"--help")==0)
	  {
	  	gnotify_about();
	  }
	  else if(strcmp(argv[1],"-v")==0||strcmp(argv[1],"--version")==0)
	  {
	  	printf("%s\n",VERSION);
			exit(0);
	  }
	}
  cfg.notify_x=0;  
  cfg.notify_y=0;  
  cfg.notify_delay=5000; 
  cfg.notify_play=1;
  cfg.notify_ipdisallows=0;
  cfg.notify_forkgtk=0;
  sprintf(buff,"%s/.gnotify.xml",getenv("HOME"));
  readconfig(buff);
  atexit(gnotify_atexit);
  signal(SIGKILL,gnotify_signal);
  signal(SIGSTOP,gnotify_signal);
  setsid();
  gnotify_main();
  exit(0);
}

void gnotify_main(void)
{
  struct sockaddr_in server;
	struct sockaddr_in client;
	int fd;
	int len;
	int i=1;
	int j=0;
	int allowed=0;
	int tth;
	
	setsid();
	if((sockett=socket(PF_INET,SOCK_STREAM,0))<0)
	{
	  fprintf(stderr,"couldn't create socket!\n");
	  exit(-1);
  }
  setsockopt(sockett,SOL_SOCKET,SO_REUSEADDR,&i,sizeof(i));
  memset(&server,0,sizeof(server));
  server.sin_family=AF_INET;
  server.sin_addr.s_addr=htonl(INADDR_ANY);
  server.sin_port = htons(7206);
	if(bind(sockett,(struct sockaddr*)&server,sizeof(server))<0)
	{
	  fprintf(stderr,"couldn't bind socket!\nMaybe GNotify is already running ...?\n");
	  exit(-1);
	}
	listen(sockett,5);
	pthread_mutex_init(&mutex, NULL);
	for(;;)
	{
    len=sizeof(client);
    if((fd=accept(sockett,(struct sockaddr*)&client, &len))<0)
		{
		  fprintf(stderr,"couldn't accept client request!\n");
    }
		else
		{
			if((tth=cid_getfree())==-1)
			{
				fprintf(stderr,"couldn't accept client request, cause: no free slot available!\n");
				close(fd);
			}
			else
			{
				strcpy(cdb[tth].ip,inet_ntoa(client.sin_addr));
				cdb[tth].sockfd=fd;
				if(cfg.notify_ipdisallows>0)
				{
					for(j=1;j<128;j++)
					{
						if(strcmp(cdb[tth].ip,cfg.notify_allowip[j])==0)
						{
							allowed=1;
							break;
						}
					}
				}
				cdb[tth].move_x=-1;
				cdb[tth].move_y=-1;
				if((cfg.notify_ipdisallows>0&&allowed==1)||(cfg.notify_ipdisallows==0))
				{
			  	if((pthread_create(&cdb[tth].th,NULL,(void *)&gnotify_client,(void*)tth))!=0)
					{
					  fprintf(stderr,"couldn't create new thread!\n");
					}
				}
				else
				{
					printf("ip '%s' is not allowed to connect!\n",cdb[tth].ip);
					close(fd);
				}
    	}
		}
  }
	pthread_exit(NULL);
}

void gnotify_atexit(void)
{
	int i;
	
	for(i=0;i<128;i++)
	{
		close(cdb[i].sockfd);
		pthread_cancel(cdb[i].th);
	}
	close(sockett);
	return;
}

void gnotify_signal(int sign)
{
	switch(sign)
	{
		case SIGKILL: gnotify_atexit(); break;
		case SIGSTOP: gnotify_atexit(); break;
	}
}

int gnotify_client(int cid)
{
  int i;
  int ffd;
  int sockfd=cdb[cid].sockfd;
  char cmd[1024];
	pthread_t gtkf;
	
	strcpy(cdb[cid].app,"GNotify");
	strcpy(cdb[cid].icon,GNOTIFY_ICON);
	strcpy(cdb[cid].button,"<NULL>");
	printf("new client connected: %s\n",cdb[cid].ip);
  do
	{
	  memset(cmd,0x00,sizeof(cmd));
		i=recv(sockfd,cmd,1022,0);
		cmd[strlen(cmd)]='\0';
		if(strncmp(cmd,"APP:",4)==0)
    {
      strcpy(cdb[cid].app,reverse_strnget(cmd,strlen(cmd)-4));
  	  memset(cmd,0x00,sizeof(cmd));
      sprintf(cmd,"220 Ok\n");
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"ICON:",5)==0)
    {
      strcpy(cdb[cid].icon,reverse_strnget(cmd,strlen(cmd)-5));
  	  memset(cmd,0x00,sizeof(cmd));
    	if(strncmp(cdb[cid].icon,"<NULL>",6)!=0&&(ffd=open(cdb[cid].icon,O_RDONLY))<0)
    	{
    		strcpy(cdb[cid].icon,GNOTIFY_ICON);
    		sprintf(cmd,"500 Icon does not exist\n");
    	}
    	else
    	{
	      sprintf(cmd,"220 Ok\n");
	    }
	    close(ffd);
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"SND:",4)==0)
    {
      strcpy(cdb[cid].snd,reverse_strnget(cmd,strlen(cmd)-5));
  	  memset(cmd,0x00,sizeof(cmd));
    	if((ffd=open(cdb[cid].snd,O_RDONLY))<0)
    	{
    		strcpy(cdb[cid].snd,"");
    		sprintf(cmd,"500 Soundfile does not exist\n");
    	}
    	else
    	{
	      sprintf(cmd,"220 Ok\n");
	    }
	    close(ffd);
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"BTN:",4)==0)
    {
      strncpy(cdb[cid].button,reverse_strnget(cmd,strlen(cmd)-4),32);
  	  memset(cmd,0x00,sizeof(cmd));
      sprintf(cmd,"220 Ok\n");
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"MVX:",4)==0)
    {
			if(cfg.notify_allowmove==1)
			{
	      cdb[cid].move_x=atoi(reverse_strnget(cmd,strlen(cmd)-4));
  		  memset(cmd,0x00,sizeof(cmd));
    	  sprintf(cmd,"220 Ok\n");
			}
			else
			{
				sprintf(cmd,"500 Moving is not allowed!\n");
			}
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"MVY:",4)==0)
    {
			if(cfg.notify_allowmove==1)
			{
	      cdb[cid].move_y=atoi(reverse_strnget(cmd,strlen(cmd)-4));
  		  memset(cmd,0x00,sizeof(cmd));
    	  sprintf(cmd,"220 Ok\n");
			}
			else
			{
				sprintf(cmd,"500 Moving is not allowed!\n");
			}
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
		else if(strncmp(cmd,"MSG:",4)==0)
    {
      strcpy(cdb[cid].msg,reverse_strnget(cmd,strlen(cmd)-4));
			if(cfg.notify_forkgtk==0)
			{
	      gnotify_notify(cid);
			}			
			else if(cfg.notify_forkgtk==1)
			{
				switch(fork())
				{
					case -1: fprintf(stderr,"couldn't fork gtk-notify\n"); break;
					case 0: gnotify_notify(cid); break;
					default: break;
				}
			}
			else if(cfg.notify_forkgtk==2)
			{
		  	if((pthread_create(&gtkf,NULL,(void *)&gnotify_notify,(void*)cid))!=0)
				{
					fprintf(stderr,"couldn't create gtk-notify-thread\n");
				}
			}
  	  memset(cmd,0x00,sizeof(cmd));
      sprintf(cmd,"220 Ok\n");
      send(sockfd,cmd,strlen(cmd)+1,0);
    }
    else if(strncmp(cmd,"VERSION",4)==0)
    {
			sprintf(cmd,"214 %s\n",VERSION);
			send(sockfd,cmd,strlen(cmd)+1,0);
    }
    else if(strncmp(cmd,"QUIT",4)==0)
    {
      break;
    }
		else
		{
			sprintf(cmd,"510 Unrecognized command\n");
			send(sockfd,cmd,strlen(cmd)+1,0);
		}
	}while(i>0);
  close(sockfd);
	printf("client disconnected!\n");
	pthread_exit((void *)0);
}

int playsound(const char *file)
{
	char buff[512];
	int i;
	/* I know, I shouldn't use such a quick 'n' dirty sound-hack... */
	if(strcmp(cfg.notify_ssink,"")==0)
	{
		g_printerr("no gstreamer-sink set! setting esdsink...");
		strcpy(cfg.notify_ssink,"esdsink");
	}
	sprintf(buff,"gst-launch filesrc location=\"%s\" ! spider ! %s",file,cfg.notify_ssink);
	i=system(buff);
	pthread_exit((void *)i);
}

int cid_getfree(void)
{
	int i;
	
	for(i=0;i<128;i++)
	{
		if(strcmp(cdb[i].ip,"")==0)
			return(i);
	}
	return(-1);
}

char *reverse_strnget(const char* string1, int n)
{
  static char buffer[1024];

  strcpy(buffer, &string1[strlen(string1)-n]);
	return(buffer);
}

int readconfig(char *config)
{
  xmlDocPtr conffile;
	xmlNodePtr cur;

	conffile = xmlParseFile(config);
	if(conffile==NULL)
  {
		printf("[xml] couldn't parse config-file successfully!\ntrying to (re)create it...\n");
		exit(createconfig(config));
	}
	cur = xmlDocGetRootElement(conffile);
	if(cur==NULL)
	{
		printf("[xml] couldn't parse config-file: empty document!");
		xmlFreeDoc(conffile);
		exit(-1);
	}
	if(xmlStrcmp(cur->name,(const xmlChar *)"gnotify"))
	{
		printf("[xml] config-file error: rootnode != \"gnotify\"");
		xmlFreeDoc(conffile);
		exit(-1);
	}
	cur=cur->xmlChildrenNode;
	while(cur!=NULL)
	{
	  if((!xmlStrcmp(cur->name,(const xmlChar *)"position_x")))
		{
		  cfg.notify_x=atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1));
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"position_y")))
		{
		  cfg.notify_y=atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1));
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"notify_delay")))
		{
		  cfg.notify_delay=atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1));
		  if(cfg.notify_delay<1000)
		  {
		  	printf("setting the notify-delay lower than 1000ms is senseless...\n");
		  }
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"playsound")))
		{
		  cfg.notify_play=atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1));
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"forkgtk")))
		{
		  cfg.notify_forkgtk=atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1));
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"threadgtk")))
		{
			if(atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1))==1)
	  		cfg.notify_forkgtk=2;
			else if(atoi(xmlNodeListGetString(conffile,cur->xmlChildrenNode,1))==0)
	  		cfg.notify_forkgtk=0;
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"gstreamer_sink")))
		{
		  strncpy(cfg.notify_ssink,xmlNodeListGetString(conffile,cur->xmlChildrenNode,1),8);
		}
	  else if((!xmlStrcmp(cur->name,(const xmlChar *)"allow_ip")))
		{
			cfg.notify_ipdisallows++;
		  strncpy(cfg.notify_allowip[cfg.notify_ipdisallows],xmlNodeListGetString(conffile,cur->xmlChildrenNode,1),8);
		}
  	cur=cur->next;
	}
	xmlFreeDoc(conffile);
	return(0);
}

int createconfig(char *config)
{
	FILE *fle;
	
	if((fle=fopen(config,"w"))==NULL)
	{
		printf("couldn't create '%s' successfully!\n",config);
		return(-1);
	}
	fprintf(fle,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<gnotify>\n");
		fprintf(fle,"\t<position_x>0</position_x>\n");
		fprintf(fle,"\t<position_y>0</position_y>\n");
		fprintf(fle,"\t<notify_delay>5000</notify_delay>\n");
		fprintf(fle,"\t<playsound>1</playsound>\n");
		fprintf(fle,"\t<gstreamer_sink>esdsink</gstreamer_sink>\n");
		fprintf(fle,"\t<!--\n\tuncomment this to allow\n\tonly connections from 127.0.0.1\n\t<allow_ip>127.0.0.1</allow_ip>\n\t-->\n");
		fprintf(fle,"\t<!--\n\tuncomment this to allow\n\tforking of the gtk-function.\n\tThat would provide simultaneous\n\taccess to the GTK-functions\n\tsothat clients will not have to\n\twait for the termination of\n\tone function call...\n\t<forkgtk>1</forkgtk>\n\t-->\n");
		fprintf(fle,"\t<!--\n\tuncomment this to allow\n\tthreading of the gtk-function.\n\tThat would provide the same\n\twhat forking provides but it\n\tis more resource-friendlier.\n\t<threadgtk>1</threadgtk>\n\t-->\n");
		fprintf(fle,"\t<allowmove>0</allowmove>\n");
	fprintf(fle,"</gnotify>");
	fclose(fle);
	printf("'%s' created successfully!\n",config);
	return(0);
}
